import { Edge } from "../graph.interface";
import { Citation } from "./citation";
import { Member } from "../member/member";
import { Post } from "../post/post";
import { ApolloTestServer } from "../../../../util/apollo-test-server";

describe('Rep Service Citation - Unit Tests', () => {

    let testingPostA: Post;
    let testingPostB: Post;
    let testingCitation: Citation;

    const TEST_AUTHOR_A = new Member("testAuthorA");
    const TEST_TITLE_A = "The Hitchhiker's Guide to the Galaxy";
    const TEST_CONTENT_A = "The ships hung in the sky in much the same way that bricks don't.";

    const TEST_AUTHOR_B = new Member("testAuthorB");
    const TEST_TITLE_B = "The Restaurant at the End of the Universe";
    const TEST_CONTENT_B = "The story so far: In the beginning the Universe was created. This has made a lot of people very angry and been widely regarded as a bad move.";

    beforeEach(() => {
        testingPostA = new Post("testingPostA", TEST_TITLE_A, TEST_CONTENT_A, TEST_AUTHOR_A);
        testingPostB = new Post("testingPostB", TEST_TITLE_B, TEST_CONTENT_B, TEST_AUTHOR_B);
        testingCitation = new Citation("testingCitation", testingPostA, testingPostB, 10);
    });

    afterAll(() => {
        Member.clear();
        Post.clear();
        Citation.clear();
    });

    it('should be able to get all citations', () => {
        const citations = Citation.getAllCitations();
        expect(citations.length).toBe(1);
    });

    it('should be able to get a specific citation', () => {
        const citation = Citation.getCitation("testingCitation");
        expect(citation).toBeDefined();
        expect(citation?.id).toBe("testingCitation");
    });

    it('should be able to create a new citation with random id', () => {
        const citation = new Citation(null, testingPostA, testingPostB, 10);
        expect(typeof citation.id).toBe("string");
        expect(citation.id.length).toBeGreaterThan(5);
    });

    it('should be able to create a new citation with specific id', () => {
        const citation = new Citation("testing", testingPostA, testingPostB, 10);
        expect(citation.id).toBe("testing");
    });

    it('should be able to get the id of a citation', () => {
        expect(testingCitation.id).toBe("testingCitation");
    });

    it('should be able to get if the citation is directional', () => {
        expect(testingCitation.directional).toBe(true);
    });

    it('should be able to get the sourcePost from a citation', () => {
        expect(testingCitation.sourcePost).toBe(testingPostA);
    });

    it('should be able to get the citedPost from a citation', () => {
        expect(testingCitation.citedPost).toBe(testingPostB);
    });

    it('should be able to get the impact of a citation', () => {
        expect(testingCitation.impact).toBe(10);
    });

    it('should be able to get if a citation is neutral', () => {
        const citation = new Citation("testing", testingPostA, testingPostB, 0);
        expect(citation.isNeutral()).toBe(true);
    });

    it('should be able to get if a citation is positive', () => {
        const citation = new Citation("testing", testingPostA, testingPostB, 10);
        expect(citation.isPositive()).toBe(true);
    });

    it('should be able to get if a citation is negative', () => {
        const citation = new Citation("testing", testingPostA, testingPostB, -10);
        expect(citation.isNegative()).toBe(true);
    });

    it('should be able to get the parentVertex from Citation as an Edge', () => {
        const edge = testingCitation as Edge;
        expect(edge.parentVertex).toBeDefined();
        expect(edge.parentVertex).toBe(testingPostB);
    });

    it('should be able to get the childVertex from Citation as an Edge', () => {
        const edge = testingCitation as Edge;
        expect(edge.childVertex).toBeDefined();
        expect(edge.childVertex).toBe(testingPostA);
    });

    it('should be able to get the adjacent Vertex from Citation as an Edge', () => {
        const edge = testingCitation as Edge;
        expect(edge.getAdjacent(testingPostA)).toBe(testingPostB);
        expect(edge.getAdjacent(testingPostB)).toBe(testingPostA);
    });
    
});


describe ('Rep Service Citation - Integration Tests', () => {

    process.env.TEST_SUITE = "Rep Service Citation - Integration Tests";
    let testServer: ApolloTestServer;

    let createUser1 = `createMember(id: "testingUser1") { id }`;
    let createUser2 = `createMember(id: "testingUser2") { id }`;
    let createPost1 = `createPost(id: "testingPost1", title: "Test Post 1", content: "Test Content 1", authorId: "testingUser1") { id }`;
    let createPost2 = `createPost(id: "testingPost2", title: "Test Post 2", content: "Test Content 2", authorId: "testingUser2") { id }`;
    let createCitation = `createCitation(citationId: "testingCitation1", sourcePostId: "testingPost1", citedPostId: "testingPost2", impact: 10) { id }`;
    
    beforeEach(async () => {
        testServer = new ApolloTestServer();
        await testServer.init();
        await testServer.executeMutation(createUser1);
        await testServer.executeMutation(createUser2);
        await testServer.executeMutation(createPost1);
        await testServer.executeMutation(createPost2);
        await testServer.executeMutation(createCitation);
    });

    afterAll(async () => {
        await testServer.close();
    });

    it('should be able to query all citations', async () => {
        const query = `citations { id }`;
        const response = await testServer.executeQuery(query);
        expect(response.data.citations.length).toBe(1);
    });

    it('should be able to query a specific citation', async () => {
        const query = `citation(id: "testingCitation1") { id }`;
        const response = await testServer.executeQuery(query);
        expect(response.data.citation.id).toBe("testingCitation1");
    });

    it('should include all required fields and cascade through types', async () => {
        const query = `citation(id: "testingCitation1") { id, directional, impact, 
            sourcePost { id, title, authors { id, reputation, ledger { balance }} },}`;
        const response = await testServer.executeQuery(query);
        const citation = response.data.citation;
        const citationFields = ["id", "directional", "impact", "sourcePost"];
        for (const field of citationFields) expect(response.data.citation[field]).toBeDefined();
        const post = citation.sourcePost;
        const postFields = ["id", "title", "authors"];
        for (const field of postFields) expect(post[field]).toBeDefined();
        const member = post.authors[0];
        const memberFields = ["id", "reputation", "ledger"];
        for (const field of memberFields) expect(member[field]).toBeDefined();
        
    });

    it('should be able to query the adjacent post on a citation', async () => {
        const query = `adjacentPost(citationId: "testingCitation1", postId: "testingPost1") { id }`;
        const response = await testServer.executeQuery(query);
        expect(response.data.adjacentPost.id).toBe("testingPost2");
    });

    it('should be able to query the sign of citations', async () => {
        const positive = await testServer.executeQuery(`isPositive(citationId: "testingCitation1")`);
        const negative = await testServer.executeQuery(`isNegative(citationId: "testingCitation1")`);
        const neutral = await testServer.executeQuery(`isNeutral(citationId: "testingCitation1")`);
        expect(positive.data.isPositive).toBe(true);
        expect(negative.data.isNegative).toBe(false);
        expect(neutral.data.isNeutral).toBe(false);
    });

    it('should be able to create a new citation', async () => {
        const query = `createCitation(citationId: "newCitation", sourcePostId: "testingPost1", citedPostId: "testingPost2", impact: -10) { id }`;
        const response = await testServer.executeMutation(query);
        expect(response.data.createCitation.id).toBe("newCitation");
    });

});


describe ('Rep Service Citation - E2E Tests', () => {

});