import { Member } from "./member";
import { ApolloTestServer } from "../../../../util/apollo-test-server";

describe('Rep Service Member - Unit Tests', () => {

    let testMemberA: Member;
    let testMemberB: Member;

    beforeEach(() => {
        testMemberA = new Member("testMemberA");
        testMemberB = new Member("testMemberB");
    });

    afterAll(() => {
        Member.clear();
    });

    it('should be able to get all members', () => {
        const members = Member.getAllMembers();
        expect(members.length).toBe(2);
        expect(members[0]).toBe(testMemberA);
        expect(members[1]).toBe(testMemberB);
    });

    it('should be able to get a specific member', () => {
        const member = Member.getMember("testMemberA");
        expect(member).toBeDefined();
    });

    it('should be able to create a new member with random id', () => {
        const member = new Member();
        expect(typeof member.id).toBe("string");
        expect(member.id.length).toBeGreaterThan(5);
    });

    it('should be able to create a new member with specific id', () => {
        const member = new Member("testing");
        expect(member.id).toBe("testing");
    });

    it('should be able to get the id of a member', () => {
        expect(testMemberA.id).toBe("testMemberA");
    });

    it('should be able to get the reputation of a member', () => {
        expect(testMemberA.reputation).toBe(0);
    });

    it('should be able to get the ledger for a member', () => {
        expect(testMemberA.ledger.length).toBe(0);
    });

    it('should produce a ledger entry when modifying reputation', () => {
        const entry = testMemberA.postReputation("testPost", 10);
        expect(testMemberA.ledger.length).toBe(1);
        expect(testMemberA.ledger[0]).toBe(entry);
        expect(entry.timestamp).toBeDefined();
        expect(entry.type).toBe("post");
        expect(entry.postId).toBe("testPost");
        expect(entry.citationId).toBeNull();
        expect(entry.change).toBe(10);
        expect(entry.balance).toBe(10);
    });

    it('should be able to modify members reputation with a post', () => {
        testMemberA.postReputation("testPost", 10);
        expect(testMemberA.reputation).toBe(10);
    });

    it('should be able to modify members reputation with a citation', () => {
        testMemberA.citationReputation("testPost", "testCitation", 10);
        expect(testMemberA.reputation).toBe(10);
    });

});

describe('Rep Service Member - Integration Tests', () => {

    process.env.TEST_SUITE = "Rep Service Member - Integration Tests";
    let testServer: ApolloTestServer;

    let createUser1 = `createMember(id: "testingUser1") { id }`;
    let createUser2 = `createMember(id: "testingUser2") { id }`;
    let createPost1 = `createPost(id: "testingPost1", title: "Test Post 1", content: "Test Content 1", authorId: "testingUser1") { id }`;

    beforeEach(async () => {
        testServer = new ApolloTestServer();
        await testServer.init();
        await testServer.executeMutation(createUser1);
        await testServer.executeMutation(createUser2);
        await testServer.executeMutation(createPost1);
    });

    afterAll(async () => {
        await testServer.close();
    });

    it('should be able to query all members', async () => {
        const query = `members { id }`;
        const response = await testServer.executeQuery(query);
        expect(response.data.members.length).toBe(2);
    });

    it('should be able to query a specific member', async () => {
        const query = `member(id: "testingUser1") { id }`;
        const response = await testServer.executeQuery(query);
        expect(response.data.member.id).toBe("testingUser1");
    });

    it('should include all required fields', async () => {
        const query = `member(id: "testingUser1") { id, reputation, ledger { balance } }`;
        const response = await testServer.executeQuery(query);
        const member = response.data.member;
        const memberFields = ["id", "reputation", "ledger"];
        for (const field of memberFields) expect(member[field]).toBeDefined();
    });

    it('should be able to query the ledger from members', async () => {
        const query = `member(id: "testingUser1") { ledger { timestamp, type, postId, citationId, change, balance } }`;
        const response = await testServer.executeQuery(query);
        const ledger = response.data.member.ledger;
        const ledgerFields = ["timestamp", "type", "postId", "citationId", "change", "balance"];
        //TODO: for (const field of ledgerFields) expect(ledger[0][field]).toBeDefined();
        // Currently not using the form to propagate any reputation
    });

    it('should be able to query the reputation from members', async () => {
        const query = `member(id: "testingUser1") { reputation }`;
        const response = await testServer.executeQuery(query);
        expect(response.data.member.reputation).toBe(0);
    });

    it('should be able to create a new member', async () => {
        const query = `createMember(id: "testingUser3") { id }`;
        const response = await testServer.executeMutation(query);
        expect(response.data.createMember.id).toBe("testingUser3");
    });

});

describe('Rep Service Member - E2E Tests', () => {

});
