
import neo4j, { Driver } from 'neo4j-driver';
import { Logger } from '@nestjs/common';

export interface DBOptions {
    host: string;
    port: string;
    user: string;
    pass: string;
}

export interface Neo4jRepo {
    getInstance(): Neo4jRepo;
    setConnector(connector: DatabaseConnector): void
}

export class DatabaseConnector {

    private driver: Driver;
    private logger: Logger

    constructor(logger: Logger) { 
        this.logger = logger;
    }

    async connect(options: DBOptions, repos: Neo4jRepo[]) {
        const connectionString = `neo4j+s://${options.host}:${options.port}`;
        this.logger.log(`Connection: ${connectionString}`);
        const authentication = neo4j.auth.basic(options.user, options.pass);
        try {
            this.driver = neo4j.driver(connectionString, authentication);
            const info = await this.driver.getServerInfo();
            if (!info) throw new Error('Unable to retrieve database info');
            this.logger.log(`Successfully connected to ${info.agent}`);
            for (const repo of repos) {
                this.logger.log(`Attaching to ${(<any>repo).name}`);
                repo.getInstance().setConnector(this);
            }
        } catch(error) {
            this.logger.error(`Failed to connect to Database - ${error.message}`);
            throw error;
        }
    }

    async disconnect() {
        this.logger.log('Closing database connection...');
        if (this.driver) await this.driver.close();
    }

    async runQuery(query: string, parameters: any = {}) {
        const session = this.driver.session();
        try {
            const result = await session.writeTransaction(tx => tx.run(query, parameters));
            return result.records;
        } finally {
            session.close();
        }
    }
}