
import { Logger } from "@nestjs/common";
import mongoose, { Document, Model, Schema } from "mongoose";

export interface LedgerModel extends Document {
    _id: string
    timestamp: string;
    type: string;
    postId: string;
    postAuthorId: string;
    citationId: string;
    citationAuthorId: string;
    change: number;
    balance: number;
}

export interface LedgerOptions {
    host: string;
    user: string;
    pass: string;
}

export class LedgerConnector {

    private static instance: LedgerConnector;
    private logger: Logger;
    private ledgerModel: Model<LedgerModel>;

    private LedgerSchema = new Schema({
        _id: String,
        timestamp: String,
        type: String,
        postId: String,
        postAuthorId: String,
        citationId: String,
        citationAuthorId: String,
        change: Number,
        balance: Number
    });

    private constructor(logger: Logger) {
        this.logger = logger;
    }

    public static getInstance(logger: Logger) {
        if (!LedgerConnector.instance) LedgerConnector.instance = new LedgerConnector(logger);
        return LedgerConnector.instance;
    }

    public async connect(options: LedgerOptions) {
        const connectionString = `mongodb+srv://${options.user}:${options.pass}@${options.host}`;
        this.logger.log(`Connecting to the Ledger @ ${options.host} as ${options.user}...`);
        try {
            await mongoose.connect(connectionString, {});
            this.logger.log('Successfully connected to the Ledger');
            this.ledgerModel = mongoose.model('Ledger', this.LedgerSchema) as Model<LedgerModel>;
        } catch(error) {
            this.logger.error(`Failed to connect to Ledger - ${error.message}`);
            throw error;
        }
    }

    public async disconnect() {
        this.logger.log('Closing Ledger connection...');
        await mongoose.disconnect();
    }

    public async addEntry(entry: LedgerModel): Promise<LedgerModel> {
        return await this.ledgerModel.create(entry);
    }

    public async get(id: string): Promise<LedgerModel> {
        return await this.ledgerModel.findById(id);
    }

    public async getByPostId(postId: string): Promise<LedgerModel[]> {
        return await this.ledgerModel.find({ postId });
    }

    public async getByCitationId(citationId: string): Promise<LedgerModel[]> {
        return await this.ledgerModel.find({ citationId });
    }

    public async getByPostAuthorId(postAuthorId: string): Promise<LedgerModel[]> {
        return await this.ledgerModel.find({ postAuthorId });
    }

    public async getCitationAuthorId(citationAuthorId: string): Promise<LedgerModel[]> {
        return await this.ledgerModel.find({ citationAuthorId });
    }

}